# -*- coding: utf-8 -*-
all_events = [
    "EVENT_PEER_ATTACH",
    "EVENT_PEER_DETACH",
    "EVENT_VOLUME_CREATE",
    "EVENT_VOLUME_START",
    "EVENT_VOLUME_STOP",
    "EVENT_VOLUME_DELETE",
    "EVENT_VOLUME_SET",
    "EVENT_VOLUME_RESET",
    "EVENT_BRICK_RESET_START",
    "EVENT_BRICK_RESET_COMMIT",
    "EVENT_BRICK_REPLACE",
    "EVENT_GEOREP_CREATE",
    "EVENT_GEOREP_START",
    "EVENT_GEOREP_STOP",
    "EVENT_GEOREP_PAUSE",
    "EVENT_GEOREP_RESUME",
    "EVENT_GEOREP_DELETE",
    "EVENT_GEOREP_CONFIG_SET",
    "EVENT_GEOREP_CONFIG_RESET",
    "EVENT_BITROT_ENABLE",
    "EVENT_BITROT_DISABLE",
    "EVENT_BITROT_SCRUB_THROTTLE",
    "EVENT_BITROT_SCRUB_FREQ",
    "EVENT_BITROT_SCRUB_OPTION",
    "EVENT_BITROT_SCRUB_ONDEMAND",
    "EVENT_QUOTA_ENABLE",
    "EVENT_QUOTA_DISABLE",
    "EVENT_QUOTA_SET_USAGE_LIMIT",
    "EVENT_QUOTA_SET_OBJECTS_LIMIT",
    "EVENT_QUOTA_REMOVE_USAGE_LIMIT",
    "EVENT_QUOTA_REMOVE_OBJECTS_LIMIT",
    "EVENT_QUOTA_ALERT_TIME",
    "EVENT_QUOTA_SOFT_TIMEOUT",
    "EVENT_QUOTA_HARD_TIMEOUT",
    "EVENT_QUOTA_DEFAULT_SOFT_LIMIT",
    "EVENT_SNAPSHOT_CREATED",
    "EVENT_SNAPSHOT_CREATE_FAILED",
    "EVENT_SNAPSHOT_ACTIVATED",
    "EVENT_SNAPSHOT_ACTIVATE_FAILED",
    "EVENT_SNAPSHOT_DEACTIVATED",
    "EVENT_SNAPSHOT_DEACTIVATE_FAILED",
    "EVENT_SNAPSHOT_SOFT_LIMIT_REACHED",
    "EVENT_SNAPSHOT_HARD_LIMIT_REACHED",
    "EVENT_SNAPSHOT_RESTORED",
    "EVENT_SNAPSHOT_RESTORE_FAILED",
    "EVENT_SNAPSHOT_DELETED",
    "EVENT_SNAPSHOT_DELETE_FAILED",
    "EVENT_SNAPSHOT_CLONED",
    "EVENT_SNAPSHOT_CLONE_FAILED",
    "EVENT_SNAPSHOT_CONFIG_UPDATED",
    "EVENT_SNAPSHOT_CONFIG_UPDATE_FAILED",
    "EVENT_SNAPSHOT_SCHEDULER_INITIALISED",
    "EVENT_SNAPSHOT_SCHEDULER_INIT_FAILED",
    "EVENT_SNAPSHOT_SCHEDULER_ENABLED",
    "EVENT_SNAPSHOT_SCHEDULER_ENABLE_FAILED",
    "EVENT_SNAPSHOT_SCHEDULER_DISABLED",
    "EVENT_SNAPSHOT_SCHEDULER_DISABLE_FAILED",
    "EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_ADDED",
    "EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_ADD_FAILED",
    "EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_EDITED",
    "EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_EDIT_FAILED",
    "EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_DELETED",
    "EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_DELETE_FAILED",
    "EVENT_SVC_MANAGER_FAILED",
    "EVENT_SVC_RECONFIGURE_FAILED",
    "EVENT_SVC_CONNECTED",
    "EVENT_SVC_DISCONNECTED",
    "EVENT_PEER_STORE_FAILURE",
    "EVENT_PEER_RPC_CREATE_FAILED",
    "EVENT_PEER_REJECT",
    "EVENT_PEER_CONNECT",
    "EVENT_PEER_DISCONNECT",
    "EVENT_PEER_NOT_FOUND",
    "EVENT_UNKNOWN_PEER",
    "EVENT_BRICK_START_FAILED",
    "EVENT_BRICK_STOP_FAILED",
    "EVENT_BRICK_DISCONNECTED",
    "EVENT_BRICK_CONNECTED",
    "EVENT_BRICKPATH_RESOLVE_FAILED",
    "EVENT_NOTIFY_UNKNOWN_OP",
    "EVENT_QUORUM_LOST",
    "EVENT_QUORUM_REGAINED",
    "EVENT_REBALANCE_START_FAILED",
    "EVENT_REBALANCE_STATUS_UPDATE_FAILED",
    "EVENT_IMPORT_QUOTA_CONF_FAILED",
    "EVENT_IMPORT_VOLUME_FAILED",
    "EVENT_IMPORT_BRICK_FAILED",
    "EVENT_COMPARE_FRIEND_VOLUME_FAILED",
    "EVENT_NFS_GANESHA_EXPORT_FAILED",
    "EVENT_EC_MIN_BRICKS_NOT_UP",
    "EVENT_EC_MIN_BRICKS_UP",
    "EVENT_GEOREP_FAULTY",
    "EVENT_GEOREP_CHECKPOINT_COMPLETED",
    "EVENT_GEOREP_ACTIVE",
    "EVENT_GEOREP_PASSIVE",
    "EVENT_QUOTA_CROSSED_SOFT_LIMIT",
    "EVENT_BITROT_BAD_FILE",
    "EVENT_CLIENT_CONNECT",
    "EVENT_CLIENT_AUTH_REJECT",
    "EVENT_CLIENT_DISCONNECT",
    "EVENT_POSIX_SAME_GFID",
    "EVENT_POSIX_ALREADY_PART_OF_VOLUME",
    "EVENT_POSIX_BRICK_NOT_IN_VOLUME",
    "EVENT_POSIX_BRICK_VERIFICATION_FAILED",
    "EVENT_POSIX_ACL_NOT_SUPPORTED",
    "EVENT_POSIX_HEALTH_CHECK_FAILED",
    "EVENT_AFR_QUORUM_MET",
    "EVENT_AFR_QUORUM_FAIL",
    "EVENT_AFR_SUBVOL_UP",
    "EVENT_AFR_SUBVOLS_DOWN",
    "EVENT_AFR_SPLIT_BRAIN",
    "EVENT_TIER_ATTACH",
    "EVENT_TIER_ATTACH_FORCE",
    "EVENT_TIER_DETACH_START",
    "EVENT_TIER_DETACH_STOP",
    "EVENT_TIER_DETACH_COMMIT",
    "EVENT_TIER_DETACH_FORCE",
    "EVENT_TIER_PAUSE",
    "EVENT_TIER_RESUME",
    "EVENT_TIER_WATERMARK_HI",
    "EVENT_TIER_WATERMARK_DROPPED_TO_MID",
    "EVENT_TIER_WATERMARK_RAISED_TO_MID",
    "EVENT_TIER_WATERMARK_DROPPED_TO_LOW",
    "EVENT_VOLUME_ADD_BRICK",
    "EVENT_VOLUME_ADD_BRICK_FAILED",
    "EVENT_VOLUME_REMOVE_BRICK_START",
    "EVENT_VOLUME_REMOVE_BRICK_START_FAILED",
    "EVENT_VOLUME_REMOVE_BRICK_COMMIT",
    "EVENT_VOLUME_REMOVE_BRICK_COMMIT_FAILED",
    "EVENT_VOLUME_REMOVE_BRICK_STOP",
    "EVENT_VOLUME_REMOVE_BRICK_STOP_FAILED",
    "EVENT_VOLUME_REMOVE_BRICK_FORCE",
    "EVENT_VOLUME_REMOVE_BRICK_FORCE_FAILED",
    "EVENT_VOLUME_REMOVE_BRICK_FAILED",
    "EVENT_VOLUME_REBALANCE_START",
    "EVENT_VOLUME_REBALANCE_STOP",
    "EVENT_VOLUME_REBALANCE_FAILED",
    "EVENT_VOLUME_REBALANCE_COMPLETE",
    "EVENT_TIER_START",
    "EVENT_TIER_START_FORCE",
    "EVENT_DHT_DISK_USAGE",
    "EVENT_DHT_INODES_USAGE",
]

PEER_ATTACH = 0
PEER_DETACH = 1
VOLUME_CREATE = 2
VOLUME_START = 3
VOLUME_STOP = 4
VOLUME_DELETE = 5
VOLUME_SET = 6
VOLUME_RESET = 7
BRICK_RESET_START = 8
BRICK_RESET_COMMIT = 9
BRICK_REPLACE = 10
GEOREP_CREATE = 11
GEOREP_START = 12
GEOREP_STOP = 13
GEOREP_PAUSE = 14
GEOREP_RESUME = 15
GEOREP_DELETE = 16
GEOREP_CONFIG_SET = 17
GEOREP_CONFIG_RESET = 18
BITROT_ENABLE = 19
BITROT_DISABLE = 20
BITROT_SCRUB_THROTTLE = 21
BITROT_SCRUB_FREQ = 22
BITROT_SCRUB_OPTION = 23
BITROT_SCRUB_ONDEMAND = 24
QUOTA_ENABLE = 25
QUOTA_DISABLE = 26
QUOTA_SET_USAGE_LIMIT = 27
QUOTA_SET_OBJECTS_LIMIT = 28
QUOTA_REMOVE_USAGE_LIMIT = 29
QUOTA_REMOVE_OBJECTS_LIMIT = 30
QUOTA_ALERT_TIME = 31
QUOTA_SOFT_TIMEOUT = 32
QUOTA_HARD_TIMEOUT = 33
QUOTA_DEFAULT_SOFT_LIMIT = 34
SNAPSHOT_CREATED = 35
SNAPSHOT_CREATE_FAILED = 36
SNAPSHOT_ACTIVATED = 37
SNAPSHOT_ACTIVATE_FAILED = 38
SNAPSHOT_DEACTIVATED = 39
SNAPSHOT_DEACTIVATE_FAILED = 40
SNAPSHOT_SOFT_LIMIT_REACHED = 41
SNAPSHOT_HARD_LIMIT_REACHED = 42
SNAPSHOT_RESTORED = 43
SNAPSHOT_RESTORE_FAILED = 44
SNAPSHOT_DELETED = 45
SNAPSHOT_DELETE_FAILED = 46
SNAPSHOT_CLONED = 47
SNAPSHOT_CLONE_FAILED = 48
SNAPSHOT_CONFIG_UPDATED = 49
SNAPSHOT_CONFIG_UPDATE_FAILED = 50
SNAPSHOT_SCHEDULER_INITIALISED = 51
SNAPSHOT_SCHEDULER_INIT_FAILED = 52
SNAPSHOT_SCHEDULER_ENABLED = 53
SNAPSHOT_SCHEDULER_ENABLE_FAILED = 54
SNAPSHOT_SCHEDULER_DISABLED = 55
SNAPSHOT_SCHEDULER_DISABLE_FAILED = 56
SNAPSHOT_SCHEDULER_SCHEDULE_ADDED = 57
SNAPSHOT_SCHEDULER_SCHEDULE_ADD_FAILED = 58
SNAPSHOT_SCHEDULER_SCHEDULE_EDITED = 59
SNAPSHOT_SCHEDULER_SCHEDULE_EDIT_FAILED = 60
SNAPSHOT_SCHEDULER_SCHEDULE_DELETED = 61
SNAPSHOT_SCHEDULER_SCHEDULE_DELETE_FAILED = 62
SVC_MANAGER_FAILED = 63
SVC_RECONFIGURE_FAILED = 64
SVC_CONNECTED = 65
SVC_DISCONNECTED = 66
PEER_STORE_FAILURE = 67
PEER_RPC_CREATE_FAILED = 68
PEER_REJECT = 69
PEER_CONNECT = 70
PEER_DISCONNECT = 71
PEER_NOT_FOUND = 72
UNKNOWN_PEER = 73
BRICK_START_FAILED = 74
BRICK_STOP_FAILED = 75
BRICK_DISCONNECTED = 76
BRICK_CONNECTED = 77
BRICKPATH_RESOLVE_FAILED = 78
NOTIFY_UNKNOWN_OP = 79
QUORUM_LOST = 80
QUORUM_REGAINED = 81
REBALANCE_START_FAILED = 82
REBALANCE_STATUS_UPDATE_FAILED = 83
IMPORT_QUOTA_CONF_FAILED = 84
IMPORT_VOLUME_FAILED = 85
IMPORT_BRICK_FAILED = 86
COMPARE_FRIEND_VOLUME_FAILED = 87
NFS_GANESHA_EXPORT_FAILED = 88
EC_MIN_BRICKS_NOT_UP = 89
EC_MIN_BRICKS_UP = 90
GEOREP_FAULTY = 91
GEOREP_CHECKPOINT_COMPLETED = 92
GEOREP_ACTIVE = 93
GEOREP_PASSIVE = 94
QUOTA_CROSSED_SOFT_LIMIT = 95
BITROT_BAD_FILE = 96
CLIENT_CONNECT = 97
CLIENT_AUTH_REJECT = 98
CLIENT_DISCONNECT = 99
POSIX_SAME_GFID = 100
POSIX_ALREADY_PART_OF_VOLUME = 101
POSIX_BRICK_NOT_IN_VOLUME = 102
POSIX_BRICK_VERIFICATION_FAILED = 103
POSIX_ACL_NOT_SUPPORTED = 104
POSIX_HEALTH_CHECK_FAILED = 105
AFR_QUORUM_MET = 106
AFR_QUORUM_FAIL = 107
AFR_SUBVOL_UP = 108
AFR_SUBVOLS_DOWN = 109
AFR_SPLIT_BRAIN = 110
TIER_ATTACH = 111
TIER_ATTACH_FORCE = 112
TIER_DETACH_START = 113
TIER_DETACH_STOP = 114
TIER_DETACH_COMMIT = 115
TIER_DETACH_FORCE = 116
TIER_PAUSE = 117
TIER_RESUME = 118
TIER_WATERMARK_HI = 119
TIER_WATERMARK_DROPPED_TO_MID = 120
TIER_WATERMARK_RAISED_TO_MID = 121
TIER_WATERMARK_DROPPED_TO_LOW = 122
VOLUME_ADD_BRICK = 123
VOLUME_ADD_BRICK_FAILED = 124
VOLUME_REMOVE_BRICK_START = 125
VOLUME_REMOVE_BRICK_START_FAILED = 126
VOLUME_REMOVE_BRICK_COMMIT = 127
VOLUME_REMOVE_BRICK_COMMIT_FAILED = 128
VOLUME_REMOVE_BRICK_STOP = 129
VOLUME_REMOVE_BRICK_STOP_FAILED = 130
VOLUME_REMOVE_BRICK_FORCE = 131
VOLUME_REMOVE_BRICK_FORCE_FAILED = 132
VOLUME_REMOVE_BRICK_FAILED = 133
VOLUME_REBALANCE_START = 134
VOLUME_REBALANCE_STOP = 135
VOLUME_REBALANCE_FAILED = 136
VOLUME_REBALANCE_COMPLETE = 137
TIER_START = 138
TIER_START_FORCE = 139
DHT_DISK_USAGE = 140
DHT_INODES_USAGE = 141
